@@ -26,6 +26,9 @@ pub mod payroll;
 pub mod quarry;
 pub mod rewarder;
 
+mod instructions;
+use instructions::*;
+
 use crate::quarry::StakeAction;
 
 declare_id!("QMNeHCGYnLVDn1icRAfQZpjPLBNkfGbSKRB83G5d8KB");
@@ -371,6 +374,15 @@ pub mod quarry_mine {
         Ok(())
     }
 
+    /// Withdraw tokens from a [Miner]-owned token account that is not the [Miner::token_vault_key].
+    /// This is useful for if tokens are sent directly to a [Miner].
+    ///
+    /// Only the [Miner::authority] may call this.
+    #[access_control(ctx.accounts.validate())]
+    pub fn rescue_tokens(ctx: Context<RescueTokens>) -> Result<()> {
+        instructions::rescue_tokens::handler(ctx)
+    }
+
     /// --------------------------------
     /// Protocol Functions
     /// --------------------------------
@@ -728,14 +740,14 @@ pub struct CreateMiner<'info> {
         payer = payer,
         space = 8 + Miner::LEN
     )]
-    pub miner: Account<'info, Miner>,
+    pub miner: Box<Account<'info, Miner>>,
 
     /// [Quarry] to create a [Miner] for.
     #[account(mut)]
-    pub quarry: Account<'info, Quarry>,
+    pub quarry: Box<Account<'info, Quarry>>,
 
     /// [Rewarder].
-    pub rewarder: Account<'info, Rewarder>,
+    pub rewarder: Box<Account<'info, Rewarder>>,
 
     /// System program.
     pub system_program: Program<'info, System>,
